#!/bin/sh
#
# Copyright (c) 2003
#	Bill Paul <wpaul@windriver.com>.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#	This product includes software developed by Bill Paul.
# 4. Neither the name of the author nor the names of any co-contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR THE VOICES IN HIS HEAD
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
#
# $FreeBSD$
#

infconv () {
clear
echo ""
echo ""
echo "			INF file validation"
echo ""
echo ""
echo "	A .INF file is most often provided as an ASCII file, however"
echo "	files with multilanguage support are provided in Unicode format."
echo "	Please type in the path to your .INF file now."
echo ""
echo -n "	> "
read INFPATH
if [ $INFPATH ] && [ -e $INFPATH ]; 
then 
	INFTYPE=`${FILE} ${INFPATH}`

	case ${INFTYPE} in
	*ASCII*)
		echo ""
		echo "	This .INF file appears to be ASCII."
		echo ""
		echo -n "	Press any key to continue... "
		read KEYPRESS
		;;
	*nicode*)
		echo ""
		echo "	This .INF file appears to be Unicode."
		if [ -e $ICONVPATH ];
		then
			echo "	Trying to convert to ASCII..."
			${ICONVPATH} -f utf-16 -t utf-8 ${INFPATH} > /tmp/ascii.inf
			INFPATH=/tmp/ascii.inf
			echo "	Done."
		else
			echo "	The iconv(1) utility does not appear to be installed."
			echo "	Please install this utility or convert the .INF file"
			echo "	to ASCII and run this utility again."
			echo ""
			exit
		fi
		;;
	*)
		echo ""
		echo "	I don't recognize this file format. It may not be a valid .INF file."
		echo ""
		echo -n "	Press enter to try again, or ^C to quit. "
		read KEYPRESS
		INFPATH=""
		;;
	esac
else
	echo ""
	echo "	The file '$INFPATH' was not found."
	echo ""
	echo -n "	Press enter to try again, or ^C to quit. "
	read KEYPRESS
	INFPATH=""
fi
}

sysconv() {
clear
echo ""
echo ""
echo "			Driver file validation"
echo ""
echo ""
echo "	Now you need to specify the name of the Windows(r) driver .SYS"
echo "	file for your device. Note that if you are running FreeBSD/amd64,"
echo "	then you must provide a driver that has been compiled for the"
echo "	64-bit Windows(r) platform. If a 64-bit driver is not available"
echo "	for your device, you must install FreeBSD/ia32 and use the"
echo "	32-bit driver instead."
echo ""
echo "	Please type in the path to the Windows(r) driver .SYS file now."
echo ""
echo -n "	> "
read SYSPATH
if [ $SYSPATH ] && [ -e $SYSPATH ];
then
	SYSTYPE=`${FILE} ${SYSPATH}`

	case ${SYSTYPE} in
	*Windows*)
		echo ""
		echo "	This .SYS file appears to be in Windows(r) PE format."
		echo ""
		echo -n "	Press any key to continue... "
		read KEYPRESS
		SYSBASE=`basename ${SYSPATH} | ${TR} '.' '_'`
		;;
	*)
		echo ""
		echo "	I don't recognize this file format. It may not be a valid .SYS file."
		echo ""

		echo -n "	Press enter to try again, or ^C to quit. "
		read KEYPRESS
		SYSPATH=""
		;;
	esac
else
	echo ""
	echo "	The file '$SYSPATH' was not found."
	echo ""
	echo -n "	Press enter to try again, or ^C to quit. "
	read KEYPRESS
	SYSPATH=""
fi 
}

ndiscvt() {
clear
echo ""
echo ""
echo "			Driver file conversion"
echo ""
echo ""
echo "	The script will now try to convert the .INF and .SYS files"
echo "	using the ndiscvt(1) utility. This utility can handle most"
echo "	.INF files, however occasionally it can fail to parse some files"
echo "	due to subtle syntax issues: the .INF syntax is very complex,"
echo "	and the Windows(r) parser will sometimes allow files with small"
echo "	syntax errors to be processed correctly which ndiscvt(1) will"
echo "	not. If the conversion fails, you may have to edit the .INF"
echo "	file by hand to remove the offending lines."
echo ""
echo -n "	Press enter to try converting the files now: "
read KEYPRESS
if ! ${NDISCVT} -i ${INFPATH} -s ${SYSPATH} -O -o ${DNAME}.h; then
	echo "CONVERSION FAILED"
	exit
else
	echo ""
	echo "	Conversion was successful."
	echo ""
	echo -n "	Press enter to continue... "
	read KEYPRESS
fi
}

drvgen() {
clear
echo ""
echo ""
echo "			Kernel module generation"
echo ""
echo ""
echo "	The script will now try to generate the kernel driver module."
echo "	This is the last step. Once this module is generated, you should"
echo "	be able to load it just like any other FreeBSD driver module."
echo ""
echo "	Press enter to compile the stub module and generate the driver"
echo -n "	module now: "
read KEYPRESS
echo ""
touch bus_if.h
touch device_if.h
echo -n "	Compiling stub... "
if ! ${CC} -D_KERNEL -DDRV_DATA_START=${SYSBASE}_drv_data_start -DDRV_NAME=${SYSBASE} -DDRV_DATA_END=${SYSBASE}_drv_data_end -I. ${STUBFILE} -c -o windrv_stub.o; then
	echo "compilation failed. Exiting."
	echo ""
	exit
else
	echo "done."
fi
echo -n	"	Linking driver... "
if ! ${LD} -Bshareable  -d -warn-common -o ${SYSBASE}.ko windrv_stub.o ${DNAME}.o; then
	echo "linking failed. Exiting."
	echo ""
	exit
else
	echo "done."
fi
echo -n "	Cleaning up... "
${RM} -f bus_if.h device_if.h windrv_stub.o
${RM} -f ${DNAME}.h ${DNAME}.o ${SYSBASE}.o
echo "done."
echo ""
echo "	The file $SYSBASE.ko has been successfully generated."
echo "	You can kldload this module to get started."
echo ""
echo -n "	Press any key to exit. "
read KEYPRESS
echo ""
echo ""
}

ICONVPATH=/usr/local/bin/iconv
NDISCVT=/usr/sbin/ndiscvt
STUBFILE=/usr/share/misc/windrv_stub.c
DNAME=windrv
OBJCOPY=/usr/bin/objcopy
CC=/usr/bin/cc
LD=/usr/bin/ld
RM=/bin/rm
TR=/usr/bin/tr
FILE=/usr/bin/file

INFPATH=""
SYSPATH=""
SYSBASE=""

clear

echo "		Windows(r) to ELF driver converter"
echo ""
echo "	This is script is designed to guide you through the process"
echo "	of converting a Windows(r) binary driver module and .INF"
echo "	specification file into a FreeBSD ELF kernel module for use"
echo "	with the NDIS compatibility system. To complete this process,"
echo "	you will need the following things:"
echo ""
echo "	- The Windows(r) driver binary file. This file has a .sys suffix"
echo "	  and should be on a driver CD provided with your network card."
echo "	- The Windows(r) .INF installation specification file that goes"
echo "	  with the driver. This should also be on the driver CD."
echo "	- The FreeBSD C compiler, cc(1) (part of the base install)."
echo "	- The FreeBSD linker, ld(1) (part of the base install)."
echo "	- The objcopy(1) utility (part of the base install)."
echo "	- The ndiscvt(1) utility (part of the base install)."
echo "	- The iconv(1) utility. This is required for converting .INF"
echo "	  files that are in Unicode format into ASCII. This is optional"
echo "	  and may not be needed if the .INF file is already in ASCII"
echo "	  format. It can be installed as an optional package."
echo ""
echo -n "			Press any key to continue... "
read KEYPRESS

clear
echo ""
echo ""
echo "	The .INF file is a text file which contains setup information"
echo "	that Windows(r) uses during the driver installation process. This"
echo "	includes a list of devices that the driver supports (PCI vendor"
echo "	and device IDs, PCCARD vendor/device strings or USB vendor/device"
echo "	codes), registry keys used to configure the driver, and a list"
echo "	of files to be installed."
echo ""
echo "	The .SYS file is the actual driver executable code. Windows(r)"
echo "	binaries are in Microsoft Portable Executable (PE) format and"
echo "	must be converted to ELF format in order to be loaded into the"
echo "	FreeBSD kernel."
echo ""
echo "	This script works in two steps. First, it needs to extract the"
echo "	device list and registry key data from the driver's .INF file"
echo "	and compile this information into a small stub module. Next,"
echo "	the .SYS file is converted into an ELF file and is linked"
echo "	together with the stub to create a FreeBSD loadable kernel"
echo "	module (.ko)."
echo ""
echo -n "			Press any key to continue... "
read KEYPRESS

while : ; do
	infconv
	if [ $INFPATH ] && [ $INFPATH != "" ]; then
		break
	fi
done

while : ; do
	sysconv
	if [ $SYSPATH ] && [ $SYSPATH != "" ]; then
		break
	fi
done

ndiscvt
drvgen
